#include "observable.h"
#include "observableDeltaPhi_Cap2.h"

using namespace std;

// #define VERBOSE


//! Constructor
ObservableDeltaPhiCapacitor2::ObservableDeltaPhiCapacitor2(const string name) :
Observable::Observable(name)
{ }


ObservableDeltaPhiCapacitor2::~ObservableDeltaPhiCapacitor2() { }



void ObservableDeltaPhiCapacitor2::verify_pools()
{
	if(!(Pools->count(string(FreeHPool)))) {
		cerr << "ObservableDeltaPhiCapacitor2::verify_pools: pool "<<FreeHPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(MembraneHPool)))) {
		cerr << "ObservableDeltaPhiCapacitor2::verify_pools: pool "<<MembraneHPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(ProteinHPool)))) {
		cerr << "ObservableDeltaPhiCapacitor2::verify_pools: pool "<<ProteinHPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(c2redPool)))) {
		cerr << "ObservableDeltaPhiCapacitor2::verify_pools: pool "<<c2redPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(c2oxPool)))) {
		cerr << "ObservableDeltaPhiCapacitor2::verify_pools: pool "<<c2oxPool<<" not defined" << endl;
		exit(-3);
	}
	
	FreeHPoolp     = getPool(FreeHPool);
	MembraneHPoolp = getPool(MembraneHPool);
	ProteinHPoolp  = getPool(ProteinHPool);
	c2redPoolp     = getPool(c2redPool);
	c2oxPoolp      = getPool(c2oxPool);
}



double ObservableDeltaPhiCapacitor2::get_value()
{
	double Q_pool = (freeHScaling *      static_cast<double>(FreeHPoolp->getInside())
					+ membraneHScaling * static_cast<double>(MembraneHPoolp->getInside())
					+ proteinHScaling *  static_cast<double>(ProteinHPoolp->getInside())
					+ c2redScaling *     static_cast<double>(c2redPoolp->getInside())
					+ c2oxScaling *      static_cast<double>(c2oxPoolp->getInside()));
	
	double Q_ges = Q_pool + Q_offset;
	
	#ifdef DEBUG
	cerr << "ObservableDeltaPhiCapacitor2::get_value: Q_ges = " << Q_ges << "  dPhi = " << Q_ges * voltagePerCharge << endl;
	#endif
	
	return(Q_ges * voltagePerCharge);
}



bool ObservableDeltaPhiCapacitor2::parseLine(char *linep)
{
	char  dummy[128];
	
	// read data into the variables
	if(sscanf(linep, "%s %s %lf %s %lf %s %lf %s %lf %s %lf %s %lf %lf", dummy, dummy, 
			  &voltagePerCharge, FreeHPool, &freeHScaling, MembraneHPool, &membraneHScaling, ProteinHPool, &proteinHScaling, c2redPool, &c2redScaling, c2oxPool, &c2oxScaling, &Q_offset) !=14) {
		cerr << "Simulation::parseFile: not enough parameters for DeltaPhi line >>" << linep << "<<" << endl;
		return(false);
	}
	// some sanity checks
	if(voltagePerCharge < 0.0) {
		cerr << "Simulation::parseFile: negative inverse capacity in DeltaPhi line >>" << linep << "<<" << endl;
		return(false);
	}

	#ifdef VERBOSE
	cout << "------------------" << endl;
	cout << "   DeltaPhi Capacitor: read: " << voltagePerCharge << endl;
	cout << "   DeltaPhi Capacitor: read: " << FreeHPool << "  " << MembraneHPool << "  " << ProteinHPool << "  " << c2redPool << "  " << c2oxPool << endl;
	cout << "   DeltaPhi Capacitor: read: " << Q_offset << endl;
	#endif
	
	return(true);
}
