#include "observable.h"
#include "observableDeltaPhi_ChemCap.h"

using namespace std;

// #define VERBOSE

//! Constructor
ObservableDeltaPhiChemCapacitor::ObservableDeltaPhiChemCapacitor(const string name) :
Observable::Observable(name)
{ }


ObservableDeltaPhiChemCapacitor::~ObservableDeltaPhiChemCapacitor() 
{ }



void ObservableDeltaPhiChemCapacitor::verify_pools()
{
	if(!(Pools->count(string(FreeHPool)))) {
		cerr << "ObservableDeltaPhiChemCapacitor::verify_pools: pool "<<HoutsidePool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(FreeHPool)))) {
		cerr << "ObservableDeltaPhiChemCapacitor::verify_pools: pool "<<FreeHPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(MembraneHPool)))) {
		cerr << "ObservableDeltaPhiChemCapacitor::verify_pools: pool "<<MembraneHPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(ProteinHPool)))) {
		cerr << "ObservableDeltaPhiChemCapacitor::verify_pools: pool "<<ProteinHPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(c2redPool)))) {
		cerr << "ObservableDeltaPhiChemCapacitor::verify_pools: pool "<<c2redPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(c2oxPool)))) {
		cerr << "ObservableDeltaPhiChemCapacitor::verify_pools: pool "<<c2oxPool<<" not defined" << endl;
		exit(-3);
	}
	
	HoutsidePoolp  = getPool(HoutsidePool);
	FreeHPoolp     = getPool(FreeHPool);
	MembraneHPoolp = getPool(MembraneHPool);
	ProteinHPoolp  = getPool(ProteinHPool);
	c2redPoolp     = getPool(c2redPool);
	c2oxPoolp      = getPool(c2oxPool);
}


double ObservableDeltaPhiChemCapacitor::get_value()
{
	double Q_pool = (freeHScaling *      static_cast<double>(FreeHPoolp->getInside())
					+ membraneHScaling * static_cast<double>(MembraneHPoolp->getInside())
					+ proteinHScaling *  static_cast<double>(ProteinHPoolp->getInside())
					+ c2redScaling *     static_cast<double>(c2redPoolp->getInside())
					+ c2oxScaling *      static_cast<double>(c2oxPoolp->getInside()));
	
	double prefactor = (d_M/(r_a * r_i)) / (4.0*M_PI * EPSILON_0 * eps);
	
	double Q_ges = Q_pool + Q_offset;
	
	#ifdef DEBUG
	cerr << "ObservableDeltaPhiChemCapacitor::get_value: Q_ges = " << Q_ges << "  dPhi = " << Q_ges*prefactor << endl;
	#endif
	
	double V_cap = Q_ges * prefactor;
	
	double deltapH = log(max(1.0, FreeHPoolp->getConcentration()/HoutsidePoolp->getConcentration()));
	double V_chem = pHVoltage * deltapH;

	return(V_cap + V_chem);
}


bool ObservableDeltaPhiChemCapacitor::parseLine(char *linep)
{
	char  dummy[128];
	
	// read data into the variables
	if(sscanf(linep, "%s %s %s %lf %lf %lf %lf %s %lf %s %lf %s %lf %s %lf %s %lf %lf", dummy, dummy, 
			  HoutsidePool, &pHVoltage, &r_a, &d_M, &eps, FreeHPool, &freeHScaling, MembraneHPool, &membraneHScaling, ProteinHPool, &proteinHScaling, c2redPool, &c2redScaling, c2oxPool, &c2oxScaling, &Q_offset) !=18) {
		cerr << "Simulation::parseFile: not enough parameters for DeltaPhi line >>" << linep << "<<" << endl;
		return(false);
	}
	
	// Simple sanity checks 
	if((r_a < 0.0) || (d_M < 0.0) || (eps < 0.0)) {
		cerr << "Simulation::parseFile: negative parameters in DeltaPhi line >>" << linep << "<<" << endl;
		return(false);
	}
	if(r_a <= d_M) {
		cerr << "Simulation::parseFile: r_a <= d_M in DeltaPhi line >>" << linep << "<<" << endl;
		return(false);
	}
	r_i = r_a - d_M;
	

	#ifdef VERBOSE
	cout << "------------------" << endl;
	cout << "   DeltaPhi ChemCapacitor: read: " << HoutsidePool << "  " << pHVoltage << endl;
	cout << "   DeltaPhi ChemCapacitor: read: " << r_a << "  " << d_M << endl;
	cout << "   DeltaPhi ChemCapacitor: read: " << eps << endl;
	cout << "   DeltaPhi ChemCapacitor: read: " << FreeHPool << "  " << MembraneHPool << "  " << ProteinHPool << "  " << c2redPool << "  " << c2oxPool << endl;
	cout << "   DeltaPhi ChemCapacitor: read: " << Q_offset << endl;
	#endif
	
	return(true);
}
