#include "observable.h"
#include "observableDeltaPhi_ChemCap2.h"

using namespace std;

// #define VERBOSE
// #define DEBUG

//! Constructor
ObservableDeltaPhiChemCapacitor2::ObservableDeltaPhiChemCapacitor2(const string name) :
Observable::Observable(name)
{ }


ObservableDeltaPhiChemCapacitor2::~ObservableDeltaPhiChemCapacitor2() 
{ }



void ObservableDeltaPhiChemCapacitor2::verify_pools()
{
	if(!(Pools->count(string(FreeHPool)))) {
		cerr << "ObservableDeltaPhiChemCapacitor2::verify_pools: pool "<<HoutsidePool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(FreeHPool)))) {
		cerr << "ObservableDeltaPhiChemCapacitor2::verify_pools: pool "<<FreeHPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(MembraneHPool)))) {
		cerr << "ObservableDeltaPhiChemCapacitor2::verify_pools: pool "<<MembraneHPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(ProteinHPool)))) {
		cerr << "ObservableDeltaPhiChemCapacitor2::verify_pools: pool "<<ProteinHPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(c2redPool)))) {
		cerr << "ObservableDeltaPhiChemCapacitor2::verify_pools: pool "<<c2redPool<<" not defined" << endl;
		exit(-3);
	}
	if(!(Pools->count(string(c2oxPool)))) {
		cerr << "ObservableDeltaPhiChemCapacitor2::verify_pools: pool "<<c2oxPool<<" not defined" << endl;
		exit(-3);
	}
	
	HoutsidePoolp  = getPool(HoutsidePool);
	FreeHPoolp     = getPool(FreeHPool);
	MembraneHPoolp = getPool(MembraneHPool);
	ProteinHPoolp  = getPool(ProteinHPool);
	c2redPoolp     = getPool(c2redPool);
	c2oxPoolp      = getPool(c2oxPool);
}


double ObservableDeltaPhiChemCapacitor2::get_value()
{
	double Q_pool = (freeHScaling *      static_cast<double>(FreeHPoolp->getInside())
					+ membraneHScaling * static_cast<double>(MembraneHPoolp->getInside())
					+ proteinHScaling *  static_cast<double>(ProteinHPoolp->getInside())
					+ c2redScaling *     static_cast<double>(c2redPoolp->getInside())
					+ c2oxScaling *      static_cast<double>(c2oxPoolp->getInside()));
	
	double Q_ges = Q_pool + Q_offset;
	
	#ifdef DEBUG
	cerr << "ObservableDeltaPhiCapacitor2::get_value: Q_ges = " << Q_ges << "  dPhi = " << Q_ges * voltagePerCharge << endl;
	#endif
	
	double V_cap = Q_ges * voltagePerCharge;
	
	// deltapH is limited to a min of 0 because there might not be any proton inside the small vesicle (=> -\infty)
	double deltapH = log10(max(1.0, FreeHPoolp->getConcentration()/HoutsidePoolp->getConcentration()));
	double V_chem = pHVoltage * deltapH;

	return(V_cap + V_chem);
}


bool ObservableDeltaPhiChemCapacitor2::parseLine(char *linep)
{
	char  dummy[128];
	
	// read data into the variables
	if(sscanf(linep, "%s %s %s %lf %lf %s %lf %s %lf %s %lf %s %lf %s %lf %lf", dummy, dummy, 
			  HoutsidePool, &pHVoltage, &voltagePerCharge, FreeHPool, &freeHScaling, MembraneHPool, &membraneHScaling, ProteinHPool, &proteinHScaling, c2redPool, &c2redScaling, c2oxPool, &c2oxScaling, &Q_offset) !=16) {
		cerr << "Simulation::parseFile: not enough parameters for DeltaPhi line >>" << linep << "<<" << endl;
		return(false);
	}
	
	// Simple sanity checks 
	if((voltagePerCharge < 0.0) || (pHVoltage < 0.0)) {
		cerr << "Simulation::parseFile: negative conversion factors in DeltaPhi_ChemCap line >>" << linep << "<<" << endl;
		return(false);
	}
	

	#ifdef VERBOSE
	cout << "------------------" << endl;
	cout << "   DeltaPhi ChemCapacitor: read: " << HoutsidePool << "  " << pHVoltage << endl;
	cout << "   DeltaPhi ChemCapacitor: read: " << voltagePerCharge << endl;
	cout << "   DeltaPhi ChemCapacitor: read: " << FreeHPool << "  " << MembraneHPool << "  " << ProteinHPool << "  " << c2redPool << "  " << c2oxPool << endl;
	cout << "   DeltaPhi ChemCapacitor: read: " << Q_offset << endl;
	#endif
	
	return(true);
}
