// Definitions for the automated inclusion into the parser
// LABEL:		OnOff_state
// CLASSNAME:	ObservableOnOffState
// 
#ifndef OBSERVABLE_ONOFFSTATE_H
#define OBSERVABLE_ONOFFSTATE_H

#include "observable.h"
#include "proteinOnOff.h"
#include "parameter.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


//! Observable that queries the state of the OnOff-protein with index idx.

//! There are no Parameters.
//! Actually, this observable is a temporary workaround to allow direct  
//! protein-protein interactions until this is implemented more directly.

class ObservableOnOffState : public Observable
{
	public:
	
	//! Constructor
	ObservableOnOffState(const std::string name);
	//! Destructor
	~ObservableOnOffState() { };

	//! The plain get_value() is not used here -> return zero.
	double get_value() { return(0.0); };
	//! Query state of a specific OnOff protein (which returns 0.0 or 1.0)
	double get_value(const long int &idx);
	
	//! Parser for line from input file. 
	bool parseLine(char *linep) {return(true);}
	
	private:
		
	//! verify that all neccessary pools are defined -> no pools here
	void verify_pools() { };

};
#endif /* OBSERVABLE_ONOFFSTATE_H */
