#include"pool.h"

using namespace std;


Pool::Pool(const double &dt) :
num_of_particles(0),
volume(1.0),
infinite(false)
{
	delta_t = dt;
}

Pool::~Pool()
{
}


Pool::Pool(long int num_of_particles_, double volume_, bool infinite_, std::string listed_, const double &dt) :
num_of_particles(num_of_particles_),
volume(volume_),
infinite(infinite_),
listed(listed_)
{
	delta_t = dt;
}

Pool::Pool(const Pool& aPool) :
num_of_particles(aPool.num_of_particles),
volume(aPool.volume),
delta_t(aPool.delta_t),
infinite(aPool.infinite)
{
}


bool Pool::isListed() 
{ 
	if(listed != "false") {
		return(true); 
	} else {
		return(false);
	}
}


bool Pool::is_empty() const
{
	if (num_of_particles == 0)
		return true;
	else
		return false;
}


double Pool::getConcentration() {
	return (static_cast<double>(num_of_particles) / volume);
}



double Pool::getOutput() const
{
	// return concentration..
	if(listed == "conc") {
		return (static_cast<double>(num_of_particles) / volume);
	}
	// .. or particle number
	else {
		return(static_cast<double>(num_of_particles));	
	}
}


void Pool::setInside(const long int &num_of_particles_) {
	num_of_particles = num_of_particles_;
}

void Pool::setInfinite(const bool &infinite_) {
	infinite = infinite_;
}


bool Pool::take_out(const double &rate)
{
	// if the pool is not empty
	if (num_of_particles > 0)
	{
		// create a random numer
		double random_number = lin_ran();
		// calculate probability for reaction
		double probability = (delta_t * rate * (num_of_particles/volume));
		
		// if random number <= probability let reaction take place
		if (random_number <= probability)
		{
			// If the Pool is not infinite remove one Particle
			if (!infinite) {
				num_of_particles--;
			}
			#ifdef DEBUG
			cout << "Pool::take_out(): took one particle out\n";
			#endif
			// return the particle
			return true;
		}
	}
	// return "no particle"
	return false;
}


bool Pool::take(const long int &num)
{
	// check if enough particles are inside and take num out
	if (num_of_particles >= num)
	{
		if (!infinite) {
			num_of_particles -= num;
		}
		return true;
	}
	
	return false;
}


// bool Pool::put_in(const double &rate)
// {
// 	// create a random number and determine probability for the reaction
// 	double random_number = lin_ran();
// 	double probability = (delta_t * rate);
// 	
// 	// if random number <= probability let the reaction take place
// 	if (random_number <= probability)
// 		{
// 		// if pool is not infinite, increase number of particles inside
// 		if (!infinite) {
// 			num_of_particles++;
// 		}
// 		return true;
// 	}
// 	return false;
// }


void Pool::put_back()
{
	if (!infinite) {
		num_of_particles++;
	}
}


void Pool::put_back(const long int &num)
{
	if (!infinite) {
		num_of_particles += num;
	}
}

