// Definitions for the automated inclusion into the parser
// Note that connectors can be specified as "connector_*" also, 
// only the first 10 chars are checked
// LABEL:		Connector
// CLASSNAME:	ProteinConnector
// 
#ifndef PROTEINCONNECTOR_H
#define PROTEINCONNECTOR_H

#include <string>
#include <stdlib.h>
#include <iostream>
#include <string>
#include <math.h>
#include "random.h"
#include "protein.h"
#include "pool.h"
#include "parameter.h"


//! Class that represents diffusion between two pools

/*!
This Class models diffusion between two pools by taking up particles from both pools 
according to the rate constants and releasing them to the respective other pool.

To be able to distinguish between different connectors, simulation::parser only checks for
"Connector" and ignores the rest of the string in the *.ves setup file. One can therefore use, 
e.g., Connector_one, Connector_two, etc, which will all instantiate this proteinConnector
with different labels.
*/ 

class ProteinConnector : public Protein
{
	
	public:
	
	//! Constructor
	ProteinConnector(const std::string name, const double &timestep);
	//! Destructor
	~ProteinConnector() { }
	
	//! Parser for setup line
	bool parseLine(char *linep);

	
	private:
	
	//! @name Internal functions (reactions) of the protein
	//@{
	//! The central reaction: take up particles and release them to the other pool
	void reaction1(Protein *p);
	//@}
	
	//! Verify that all neccessary pools are defined
	void verify_pools();
	
	//! @name Pointers to the two pools and variables for the uptake constants
	//@{
	Pool *Pool1p, *Pool2p;
	double Pool1_kon, Pool2_kon;
	//@}
	
	//! @name Char[]s for the pool names (see length above)
	//@{
	char Pool1[MAX_POOLNAME_LENGTH], 
		 Pool2[MAX_POOLNAME_LENGTH];
	//@}
		 
	// Defs of the internal registers and binding sites.
	// -> Connector has no interals or binding sites
};

#endif /* PROTEINCONNECTOR_H */
