// Definitions for the automated inclusion into the parser
// LABEL:		OnOff
// CLASSNAME:	ProteinOnOff
// 
#ifndef ProteinOnOff_h
#define ProteinOnOff_h

#include<string>
#include<stdlib.h>
#include<iostream>
#include<string>
#include"random.h"
#include"protein.h"
#include"pool.h"
#include"parameter.h"



//! Example protein to show how direct interactions between proteins could be implemented.

/*!
Class that represents a unit of a ring that changes its state depending on the neighbors' states.
The neighbors' states are queried via an observable getOnOff.

Note that this is a quick'n'dirty hack to be replaced with a more defined solution.
*/

class ProteinOnOff : public Protein
{
	
	public:
	
	//! Constructor, sets the initializing states of the binding, sites, internals ,...      
	ProteinOnOff(const std::string name, const double &timestep);
	//! Destructor
	~ProteinOnOff() { }
	
	//! Parse setup line
	bool parseLine(char *linep);
		
	//! For queries from neighbors
	double get_state() {
		return(myState?(1.0):(0.0)); 
	}
	
	

	private:
	
	//! @name Internal functions (reactions) of the protein
	//@{
	//! Only one reaction: get neighbors' state and switch on or switch off when on
	void reaction1(Protein *p);
	//@}
	
	//! Defs of the internal registers and binding sites.
	//! (registers = int,  binding sites = bool)
	// @{
	//! boolean var for state: true = on
	bool myState;
	// @}
	
	//! @name Char[]s for the (pool abd) observable names (see length above)
	//@{
	char neighborStateObs[MAX_POOLNAME_LENGTH]; 
	//@}
	
	//! Verify that all neccessary pools are defined
	void verify_pools();
	
	// std::string listInternals();
	
	//! @name Pointer to pools and variables for reaction constants
	//@{
	Observable *neighborStateObsp;
	double kon, koff, kL, kR;
	long int idxL, idxR;
	//@}
	
	//! timer variable for decay to off state
	unsigned long int timer_r1;
};
#endif
