// Definitions for the automated inclusion into the parser
// LABEL:		protonateable
// CLASSNAME:	ProteinProtonateable
// 
#ifndef ProteinProtonateable_h
#define ProteinProtonateable_h

#include<string>
#include<stdlib.h>
#include<iostream>
#include<string>
#include"random.h"
#include"protein.h"
#include"pool.h"
#include"parameter.h"



//! Class that represents protonateable residues in an H-pool.

/*!
This Class represents the protonateable residues of the membrane proteins. 
According to the pH H+ are taken out or put back into the pool until all 
specified binding sites are occupied.

Accesses the Proton-Pool and the (virtual) pool for protons bound (for electric part of dPhi)
*/

class ProteinProtonateable : public Protein
{
	
	public:
	
	//! Constructor, sets the initial states of the binding, sites, internals ,...      
	ProteinProtonateable(const std::string name, const double &timestep);
	//! Destructor
	~ProteinProtonateable() { }
		
	//! Parse setup line
	bool parseLine(char *linep);
	
	
	private:
	
	//! @name Internal functions (reactions) of the protein
	//@{
	//! determine the pH and adjust the occupancy of the protonateable groups
	void reaction1(Protein *p);
	//@}

	//! Defs of the internal registers and binding sites.
	long int occupied_sites;
	
	//! @name Char[] for the pool name (see length above)
	//@{
	char ProtonPool[MAX_POOLNAME_LENGTH],
		 HProteinPool[MAX_POOLNAME_LENGTH]; 
	//@}
	
	//! verify that all neccessary pools are defined
	void verify_pools();
	
	//! @name For listing the internals
	// @{
	std::string listInternals();
	void writeInternals();
	// @}
	
	//! @name Pointer to pools and variables for reaction constants to avoid their repeated lookup
	//@{
	Pool *ProtonPoolp, *HProteinPoolp;
	double numOfResidues, pK, dpK;
	//@}
};
#endif
